/*
 *  Rexx/Tk - Resizeable Notebook Widget
 *  Copyright (C) 2001  Mark Hessling  <M.Hessling@qut.edu.au>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "rexxtk.h"

char *RxPackageName = "rexxtknotebook";
char *ExtensionSource =
"# A Resizable Notebook widget for Tcl/Tk\n"
"# $Revision: 1.4 $\n"
"#\n"
"# Copyright (C) 1999 Daniel Roche\n"
"#\n"
"# This library is free software; you can redistribute it and/or\n"
"# modify it under the terms of the GNU Library General Public\n"
"# License as published by the Free Software Foundation; either\n"
"# version 2 of the License, or (at your option) any later version.\n"
"#\n"
"# This library is distributed in the hope that it will be useful,\n"
"# but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
"# Library General Public License for more details.\n"
"# \n"
"# You should have received a copy of the GNU Library General Public\n"
"# License along with this library; if not, write to the\n"
"# Free Software Foundation, Inc., 59 Temple Place - Suite 330,\n"
"# Boston, MA  02111-1307, USA.\n"
"#\n"
"# Author contact information:\n"
"#   daniel.roche@bigfoot.com\n"
"#   http://www.multimania.com/droche/\n"
"#\n"
"# Thanks to Earl Roosa <earl.roosa@db.com> for the addtab patch\n"
"#\n"
"#########################################################\n"
"# \n"
"# Rnotebook:create new_widget_name [option value ...]\n"
"#  where options are :\n"
"#   [-borderwidth i]      set the borderwidth of the tabs and the frame\n"
"#   [-height i]           set the height of the tabs in pixel\n"
"#   [-tabs tablist]       set the tabs according to each element of the list\n"
"#   [-nbtab i]            set the tabs number, by default the tabs are named \n"
"#                         according to their position number\n"
"#   [-padx i]             set the space between each tabs in pixel\n"
"#   [-background col]     set the background color for the tabs and the frame\n"
"#   [-foreground col]     set the background color for the tabs\n"
"#   [-command cmd]        command to run when tab is raised\n"
"#\n"
"# Rnotebook:config widget_name [option value ...]\n"
"#  where options are :\n"
"#   [-borderwidth i]      set the borderwidth of the tabs and the frame\n"
"#   [-height i]           set the height of the tabs in pixel\n"
"#   [-padx i]             set the space between each tabs in pixel\n"
"#   [-background col]     set the background color for the tabs and the frame\n"
"#   [-foreground col]     set the background color for the tabs\n"
"#   [-tabs tablist]       set the tabs according to each element of the list\n"
"#   [-nbtab i]            set the tabs number, by default the tabs are named \n"
"#                         according to their position number\n"
"#   [-command cmd]        command to run when tab is raised\n"
"#\n"
"# Rnotebook:raise widget_name page_number\n"
"#  raise the specified page number of the specified notebook\n"
"#\n"
"# Rnotebook:frame widget_name page_number\n"
"#  return the frame widget associated with the specified page number\n"
"#  it is a real tcl 'frame' widget and thus can be configured as any other frame.\n"
"#\n"
"# Rnotebook:button widget_name page_number\n"
"#  return the button widget associated with the specified page number\n"
"#  it is a real tcl 'button' widget and thus can be configured as any other button.\n"
"#\n"
"#########################################################\n"
"\n"
"\n"
"#\n"
"# Create a new notebook widget\n"
"#\n"
"proc Rnotebook:create {w args} {\n"
"    global Rnotebook tcl_platform\n"
"    # create/init base widget\n"
"    frame $w -borderwidth 0 -relief flat\n"
"    # init default data structure\n"
"    set Rnotebook($w,nbtab) 1\n"
"    set Rnotebook($w,tabs) {1}\n"
"    set Rnotebook($w,current) 0\n"
"    set Rnotebook($w,theight) 30\n"
"    set Rnotebook($w,padx) 0\n"
"    set Rnotebook($w,bwidth) 2\n"
"    set Rnotebook($w,background) [$w cget -background]\n"
"    set Rnotebook($w,foreground) black\n"
"    set Rnotebook($w,command) {}\n"
"    # parse arguments\n"
"    foreach {tag value} $args {\n"
"   switch -- $tag {\n"
"       -borderwidth {\n"
"      set Rnotebook($w,bwidth) $value\n"
"       }\n"
"       -height {\n"
"      set Rnotebook($w,theight) $value\n"
"       }\n"
"       -tabs {\n"
"      set Rnotebook($w,tabs) $value\n"
"      set Rnotebook($w,nbtab) [llength $value]\n"
"       }\n"
"       -nbtab {\n"
"      set Rnotebook($w,nbtab) $value\n"
"       }\n"
"       -padx {\n"
"      set Rnotebook($w,padx) $value\n"
"       }\n"
"       -background {\n"
"      set Rnotebook($w,background) $value\n"
"       }\n"
"       -foreground {\n"
"      set Rnotebook($w,foreground) $value\n"
"       }\n"
"       -command {\n"
"      set Rnotebook($w,command) $value\n"
"       }\n"
"   }\n"
"    }\n"
"    # build associated widgets\n"
"    frame $w.tabs -borderwidth 0 -relief flat\n"
"       #-background $Rnotebook($w,background)\n"
"    frame $w.body -borderwidth $Rnotebook($w,bwidth) -relief raised -background $Rnotebook($w,background)\n"
"    frame $w.mask -borderwidth 0 -relief flat -background $Rnotebook($w,background)\n"
"    frame $w.mskl -borderwidth 0 -relief flat -background $Rnotebook($w,background)\n"
"    if [string compare $tcl_platform(platform) windows] {\n"
"   # Unix look :-)\n"
"   frame $w.mskl.ml -borderwidth $Rnotebook($w,bwidth) -relief sunken\n"
"   place $w.mskl.ml -x -$Rnotebook($w,bwidth) -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 3 * $Rnotebook($w,bwidth)]\n"
"    } else {\n"
"   # losedoze look :-(\n"
"   frame $w.mskl.ml -borderwidth $Rnotebook($w,bwidth) -relief raised\n"
"   place $w.mskl.ml -x 0 -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 4 * $Rnotebook($w,bwidth)]\n"
"    }\n"
"    frame $w.mskr -borderwidth 0 -relief flat -background $Rnotebook($w,background)\n"
"    if [string compare $tcl_platform(platform) windows] {\n"
"   # Unix look :-)\n"
"   frame $w.mskr.mr -borderwidth $Rnotebook($w,bwidth) -relief sunken\n"
"   place $w.mskr.mr -x 0 -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 3 * $Rnotebook($w,bwidth)]\n"
"    } else {\n"
"   # losedoze look :-(\n"
"   frame $w.mskr.mr -borderwidth $Rnotebook($w,bwidth) -relief raised\n"
"   place $w.mskr.mr -x -$Rnotebook($w,bwidth) -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 4 * $Rnotebook($w,bwidth)]\n"
"    }\n"
"    place $w.tabs -x 0 -y 0 -relwidth 1.0 -height $Rnotebook($w,theight)\n"
"    place $w.body -x 0 -y $Rnotebook($w,theight) -relwidth 1.0 -relheight 1.0 -height -$Rnotebook($w,theight)\n"
"    for {set ind 1} {$ind <= $Rnotebook($w,nbtab)} {incr ind} {\n"
"        Rnotebook:addtab $w $ind\n"
"    }\n"
"    # show page 1\n"
"    eval Rnotebook:raise $w 1\n"
"    return $w\n"
"}\n"
"\n"
"proc Rnotebook:addtab {w ind} {\n"
"    global Rnotebook\n"
"    set i2 [expr $ind - 1]\n"
"    set txt [lindex $Rnotebook($w,tabs) $i2]\n"
"    if {! [string length $txt] } {\n"
"        set txt $ind\n"
"    }\n"
"    button $w.tabs.b$ind -text $txt -borderwidth $Rnotebook($w,bwidth) -background $Rnotebook($w,background) -foreground $Rnotebook($w,foreground) -command \"Rnotebook:raise $w $ind\"\n"
"    pack $w.tabs.b$ind -side left -pady 0 -padx $Rnotebook($w,padx) -fill y\n"
"    bind $w.tabs.b$ind <Configure> \"Rnotebook:raise $w current\"\n"
"    frame $w.body.f$ind -borderwidth 0 -background $Rnotebook($w,background)\n"
"\n"
"}\n"
"\n"
"#\n"
"# Change configuration options for the notebook widget\n"
"#\n"
"proc Rnotebook:config {w args} {\n"
"  global Rnotebook tcl_platform\n"
"  set nbt $Rnotebook($w,nbtab)\n"
"  foreach {tag value} $args {\n"
"    switch -- $tag {\n"
"      -borderwidth {\n"
"       set Rnotebook($w,bwidth) $value\n"
"       $w.body configure -borderwidth $Rnotebook($w,bwidth)\n"
"       $w.mskl.ml configure -borderwidth $Rnotebook($w,bwidth)\n"
"       $w.mskr.mr configure -borderwidth $Rnotebook($w,bwidth)\n"
"       set lst [winfo children $w.tabs]\n"
"       foreach wid $lst {\n"
"         $wid configure -borderwidth $Rnotebook($w,bwidth)\n"
"       }\n"
"       if [string compare $tcl_platform(platform) windows] {\n"
"         # Unix look :-)\n"
"         place $w.mskl.ml -x -$Rnotebook($w,bwidth) -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 3 * $Rnotebook($w,bwidth)]\n"
"       } else {\n"
"         # losedoze look :-(\n"
"         place $w.mskl.ml -x 0 -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 4 * $Rnotebook($w,bwidth)]\n"
"       }\n"
"       if [string compare $tcl_platform(platform) windows] {\n"
"         # Unix look :-)\n"
"         place $w.mskr.mr -x 0 -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 3 * $Rnotebook($w,bwidth)]\n"
"       } else {\n"
"         # losedoze look :-(\n"
"         place $w.mskr.mr -x -$Rnotebook($w,bwidth) -y -$Rnotebook($w,bwidth) -width [expr 2 * $Rnotebook($w,bwidth)] -height [expr 4 * $Rnotebook($w,bwidth)]\n"
"       }\n"
"     }\n"
"     -background {\n"
"      set Rnotebook($w,background) $value\n"
"      $w.body configure -background $Rnotebook($w,background)\n"
"      $w.mskl.ml configure -background $Rnotebook($w,background)\n"
"      $w.mskr.mr configure -background $Rnotebook($w,background)\n"
"      set lst [winfo children $w.tabs]\n"
"      foreach wid $lst {\n"
"        $wid configure -background $Rnotebook($w,background)\n"
"      }\n"
"      $w.mask configure -background $Rnotebook($w,background)\n"
"     }\n"
"     -foreground {\n"
"      set Rnotebook($w,foreground) $value\n"
"      set lst [winfo children $w.tabs]\n"
"      foreach wid $lst {\n"
"        $wid configure -foreground $Rnotebook($w,foreground)\n"
"      }\n"
"     }\n"
"     -height {\n"
"      set Rnotebook($w,theight) $value\n"
"      place $w.tabs -x 0 -y 0 -relwidth 1.0 -height $Rnotebook($w,theight)\n"
"      place $w.body -x 0 -y $Rnotebook($w,theight) -relwidth 1.0 -relheight 1.0 -height -$Rnotebook($w,theight)\n"
"     }\n"
"     -padx {\n"
"      set Rnotebook($w,padx) $value\n"
"      set lst [winfo children $w.tabs]\n"
"      foreach wid $lst {\n"
"        pack $wid -padx $Rnotebook($w,padx)\n"
"      }\n"
"     }\n"
"     -tabs {\n"
"      set Rnotebook($w,tabs) $value\n"
"      set Rnotebook($w,nbtab) [llength $value]\n"
"     }\n"
"     -nbtab {\n"
"      set Rnotebook($w,nbtab) $value\n"
"     }\n"
"     -command {\n"
"      set Rnotebook($w,command) $value\n"
"     }\n"
"    }\n"
"  }\n"
"  if {$nbt < $Rnotebook($w,nbtab)} {\n"
"    for {set ind [expr $nbt+1]} {$ind <= $Rnotebook($w,nbtab)} {incr ind} {\n"
"      Rnotebook:addtab $w $ind\n"
"    }\n"
"  }\n"
"}\n"
"\n"
"#\n"
"# This procedure raises a notebook page given its number.\n"
"# first page is number 1\n"
"#\n"
"proc Rnotebook:raise {w num} {\n"
"  global Rnotebook\n"
"  if { ![string compare $num \"current\"] } {\n"
"    set num $Rnotebook($w,current)\n"
"  }\n"
"  if { $num == 0 } {\n"
"    set num 1\n"
"  }\n"
"  if { $num != $Rnotebook($w,current) } {\n"
"    pack forget $w.body.f$Rnotebook($w,current)\n"
"    pack $w.body.f$num -fill both -expand 1\n"
"  }\n"
"  set Rnotebook($w,current) $num\n"
"  set bw $Rnotebook($w,bwidth)\n"
"\n"
"  set x0 [expr [winfo x $w.tabs.b$num] + [winfo x $w.tabs] + $bw ]\n"
"  set y0 [expr [winfo y $w.tabs.b$num] + [winfo y $w.tabs] + [winfo height $w.tabs.b$num] - $bw]\n"
"  set w0 [expr [winfo width $w.tabs.b$num] - ($bw * 2)] \n"
"  set h0 [expr $bw * 2]\n"
"  place $w.mask -x $x0 -y $y0 -width $w0 -height $h0\n"
"\n"
"  set x1 [expr $x0 - $bw]\n"
"  set y1 $y0\n"
"  set w1 $bw\n"
"  set h1 $h0\n"
"  place $w.mskl -x $x1 -y $y1 -width $w1 -height $h1\n"
"  \n"
"  set x2 [expr $x0 + $w0]\n"
"  set y2 $y0\n"
"  set w2 $bw\n"
"  set h2 $h0\n"
"  place $w.mskr -x $x2 -y $y2 -width $w2 -height $h2\n"
"  if { [string length $Rnotebook($w,command)] > 0} {\n"
"    eval $Rnotebook($w,command) $num\n"
"  }\n"
"}\n"
"\n"
"#\n"
"# Return the frame associated with a given page number\n"
"#\n"
"proc Rnotebook:frame {w num} {\n"
"  global Rnotebook\n"
"  set i \"$w.body.f$num\"\n"
"  if [ winfo exists $i] {\n"
"    return $i\n"
"  } else {\n"
"    return {}\n"
"  }\n"
"}\n"
"\n"
"#\n"
"# Return the button associated with a given page number\n"
"#\n"
"proc Rnotebook:button {w num} {\n"
"  global Rnotebook\n"
"  set i \"$w.tabs.b$num\"\n"
"  if [ winfo exists $i] {\n"
"    return $i\n"
"  } else {\n"
"    return {}\n"
"  }\n"
"}\n";

RexxFunctionHandler TkNotebookLoadFuncs    ;
RexxFunctionHandler TkNotebookDropFuncs    ;
RexxFunctionHandler TkNotebook  ;
RexxFunctionHandler TkNotebookAddTab  ;
RexxFunctionHandler TkNotebookButton  ;
RexxFunctionHandler TkNotebookConfig  ;
RexxFunctionHandler TkNotebookFrame  ;
RexxFunctionHandler TkNotebookRaise  ;

/*-----------------------------------------------------------------------------
 * Table of TK Functions. Used to install/de-install functions.
 * If you change this table, don't forget to change the table at the end
 * of this file.
 *----------------------------------------------------------------------------*/
RexxFunction RxPackageFunctions[] = {
   { "TKNOTEBOOKDROPFUNCS"       ,TkNotebookDropFuncs       ,"TkNotebookDropFuncs"       , 1 },
   { "TKNOTEBOOKLOADFUNCS"       ,TkNotebookLoadFuncs       ,"TkNotebookLoadFuncs"       , 0 }, /* don't load this from a DLL */
   { "TKNOTEBOOK"                ,TkNotebook                ,"TkNotebook"                , 1 },
   { "TKNOTEBOOKADDTAB"          ,TkNotebookAddTab          ,"TkNotebookAddTab"          , 1 },
   { "TKNOTEBOOKBUTTON"          ,TkNotebookButton          ,"TkNotebookButton"          , 1 },
   { "TKNOTEBOOKCONFIG"          ,TkNotebookConfig          ,"TkNotebookConfig"          , 1 },
   { "TKNOTEBOOKFRAME"           ,TkNotebookFrame           ,"TkNotebookFrame"           , 1 },
   { "TKNOTEBOOKRAISE"           ,TkNotebookRaise           ,"TkNotebookRaise"           , 1 },
   { NULL, NULL, NULL, 0 }
};

static char czTclCommand[TCLCOMMANDLEN];
static REXXTKDATA *RexxTkData;
   
#if defined(WIN32) || defined(OS2_DYN)
Tcl_Interp *RexxTk_TclCreateInterp(void)
{
   return RexxTkData->Dyn_TclCreateInterp();
}

int RexxTk_TclEval(Tcl_Interp *interp, char *string)
{
   return RexxTkData->Dyn_TclEval( interp, string );
}

int RexxTk_TclInit(Tcl_Interp *interp)
{
   return RexxTkData->Dyn_TclInit( interp );
}

int RexxTk_TkInit(Tcl_Interp *interp)
{
   return RexxTkData->Dyn_TkInit( interp );
}
#endif

/*
 * Rexx/Tk notebook functions start here...
 */

/*
 * Rnotebook:create pathName ?options?
 * TkNotebook(pathName [,options])
 */
RFH_RETURN_TYPE TkNotebook
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   return rtk_TypeA(RexxTkData,czTclCommand,name,"Rnotebook:create", argc, argv, retstr);
}

/*
 * Rnotebook:addtab pathName index
 * TkNotebookAddTab(pathName, index)
 */
RFH_RETURN_TYPE TkNotebookAddTab
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   if (RexxTkData->REXXTK_IntCode) ClearIntError( RexxTkData);

   if ( my_checkparam( name, argc, 2, 2 ) )
      return 1;

   return rtk_TypeB(RexxTkData,czTclCommand,name,"Rnotebook:addtab", argc, argv, retstr);
}

/*
 * Rnotebook:button pathName index
 * TkNotebookButton(pathName, index)
 */
RFH_RETURN_TYPE TkNotebookButton
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   if (RexxTkData->REXXTK_IntCode) ClearIntError( RexxTkData);

   if ( my_checkparam( name, argc, 2, 2 ) )
      return 1;

   return rtk_TypeB(RexxTkData,czTclCommand,name,"Rnotebook:button", argc, argv, retstr);
}

/*
 * Rnotebook:frame pathName index
 * TkNotebookFrame(pathName, index)
 */
RFH_RETURN_TYPE TkNotebookFrame
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   if (RexxTkData->REXXTK_IntCode) ClearIntError( RexxTkData);

   if ( my_checkparam( name, argc, 2, 2 ) )
      return 1;

   return rtk_TypeB(RexxTkData,czTclCommand,name,"Rnotebook:frame", argc, argv, retstr);
}

/*
 * Rnotebook:raise pathName index
 * TkNotebookRaise(pathName, index)
 */
RFH_RETURN_TYPE TkNotebookRaise
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   if (RexxTkData->REXXTK_IntCode) ClearIntError( RexxTkData);

   if ( my_checkparam( name, argc, 2, 2 ) )
      return 1;

   return rtk_TypeB(RexxTkData,czTclCommand,name,"Rnotebook:raise", argc, argv, retstr);
}

/*
 * Rnotebook:config pathName ?options?
 * TkNotebookConfig(pathName [,options])
 */
RFH_RETURN_TYPE TkNotebookConfig
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   FunctionPrologue( (char *)name, argc, argv );

   if (RexxTkData->REXXTK_IntCode) ClearIntError( RexxTkData);

   if ( my_checkparam( name, argc, 1, 0 ) )
      return 1;

   return rtk_TypeC(RexxTkData,czTclCommand,name,"Rnotebook:config", argc, argv, retstr);
}

RFH_RETURN_TYPE TkNotebookDropFuncs
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   ULONG rc=0;
   int unload=0;

   if ( my_checkparam(name, argc, 0, 1 ) )
      return( 1 );
   if ( argv[0].strlength == 6
   &&   memcmpi( argv[0].strptr, "UNLOAD", 6 ) == 0 )
      unload = 1;
   (void)TermRxPackage( RxPackageName, unload );
   return RxReturnNumber( retstr, rc );
}


RFH_RETURN_TYPE TkNotebookLoadFuncs
   (RFH_ARG0_TYPE name, RFH_ARG1_TYPE argc, RFH_ARG2_TYPE argv, RFH_ARG3_TYPE stck, RFH_ARG4_TYPE retstr)
{
   ULONG rc = 0L;

#if defined(DYNAMIC_LIBRARY)
   if ( !QueryRxFunction( "TKWAIT" ) )
   {
      fprintf(stderr,"The base Rexx/Tk function package must be loaded before this one\n");
      return RxReturnNumber( retstr, 1 );
   }
   /*
    * get the pointer to the tcl Interpreter and the base data from base Rexx/Tk
    * library
    */
   if ( argc == 0 )
   {
      fprintf(stderr,"You must pass the return value from TkGetBaseData() as the one and only argument.\n");
      return RxReturnNumber( retstr, 1 );
   }
   RexxTkData = (REXXTKDATA *)atol(argv[0].strptr);
   rc = InitRxPackage( NULL );
   /* 
    * Register all external functions
    */
   if ( !rc )
   {
      rc = RegisterRxFunctions( );
   }
#endif
   return RxReturnNumber( retstr, rc );
}
   
/*
 * The following functions are used in rxpackage.c
 */

/*-----------------------------------------------------------------------------
 * Execute any initialisation
 *----------------------------------------------------------------------------*/
int InitialisePackage

#ifdef HAVE_PROTO
   ( void )
#else
   ( )
#endif

{
   InternalTrace( "InitialisePackage", NULL );

   /*
    * Install the Tree widget
    */
   if (Tcl_Eval( RexxTkData->RexxTkInterp,ExtensionSource ) !=TCL_OK) {
      fprintf(stderr, "Tk_Eval for Rnotebook widget failed miserably at line %d: %s\n", RexxTkData->RexxTkInterp->errorLine, RexxTkData->RexxTkInterp->result);
      return 1;
   }
   DEBUGDUMP(fprintf(stderr,"%s-%d: After Tcl_Eval()\n",__FILE__,__LINE__);)
   return 0;
}

/*-----------------------------------------------------------------------------
 * Execute any termination
 *----------------------------------------------------------------------------*/
int TerminatePackage

#ifdef HAVE_PROTO
   ( void )
#else
   ( )
#endif

{
   return 0;
}


#if defined(USE_REXX6000)
/*
 * This function is used as the entry point for the REXX/6000
 * Rexx Interpreter
 * If you change this table, don't forget to change the table at the
 * start of this file.
 */
USHORT InitFunc( RXFUNCBLOCK **FuncBlock )
{
   static RXFUNCBLOCK funcarray[] =
   {
      { "TKNOTEBOOKDROPFUNCS"       ,TkNotebookDropFuncs      ,NULL },
      { "TKNOTEBOOKLOADFUNCS"       ,TkNotebookLoadFuncs      ,NULL },
      { "TKNOTEBOOK"                ,TkNotebook               ,NULL },
      { "TKNOTEBOOKADDTAB"          ,TkNotebookAddTab         ,NULL },
      { "TKNOTEBOOKBUTTON"          ,TkNotebookButton         ,NULL },
      { "TKNOTEBOOKCONFIG"          ,TkNotebookConfig         ,NULL },
      { "TKNOTEBOOKFRAME"           ,TkNotebookFrame          ,NULL },
      { "TKNOTEBOOKRAISE"           ,TkNotebookRaise          ,NULL },
      { NULL, NULL, NULL }
   } ;
   *FuncBlock = funcarray;
   return (USHORT)0;
}
#endif
